/*
 * HW_functions.c
 *
 *  Created on: 23.06.2014
 *      Author: NXP67729
 */


#include "chip.h"
#include "board.h"
#include "HW_functions.h"
#include "global_variables.h"



// sets up the Board so that the NTAG can be used
void HW_setup_Board_for_use_with_NTAG()
{
#ifdef Board_Demo_v1_4
  Chip_GPIO_SetPinDIROutput(LPC_GPIO_PORT, 0, VCC_SW);
  Chip_GPIO_SetPinDIRInput(LPC_GPIO_PORT, 0, FD_PIN);
  Chip_GPIO_SetPinState(LPC_GPIO_PORT, 0, VCC_SW, true);
#else

#endif
}



/// Switches LEDs off
static void ledsOFF()
{
#if defined(Board_Demo_v1_4) || defined(Board_11U68)
  Board_LED_Set(RED, false);
  Board_LED_Set(GREEN, false);
  Board_LED_Set(BLUE, false);
#else

#endif
}

//---------------------------------------------------------------------
///
/// Switches only red LED on
static void Red()
{
#if defined(Board_Demo_v1_4) || defined(Board_11U68)
  Board_LED_Set(RED, true);
  Board_LED_Set(GREEN, false);
  Board_LED_Set(BLUE, false);
#else

#endif
}

//---------------------------------------------------------------------
///
/// Switches only green LED on
static void Green()
{
#if defined(Board_Demo_v1_4) || defined(Board_11U68)
    Board_LED_Set(RED, false);
    Board_LED_Set(GREEN, true);
    Board_LED_Set(BLUE, false);
#else

#endif
}

//---------------------------------------------------------------------
///
/// Switches only blue LED on
static void Blue()
{
#if defined(Board_Demo_v1_4) || defined(Board_11U68)
    Board_LED_Set(RED, false);
    Board_LED_Set(GREEN, false);
    Board_LED_Set(BLUE, true);
#else

#endif
}


//---------------------------------------------------------------------
void HW_switchLEDs(LED led)
{
  if(led == LEDOFF)
    ledsOFF();
  else if(led == REDLED)
    Red();
  else if(led == GREENLED)
    Green();
  else if(led == BLUELED)
    Blue();
}

//---------------------------------------------------------------------
void HW_init_Buttons()
{
#ifdef Board_Demo_v1_4
   // Setting pins to input pull-up
   Chip_GPIO_SetPinState(LPC_GPIO_PORT, 0, Button1, false);
   Chip_GPIO_SetPinState(LPC_GPIO_PORT, 0, Button2, false);
   Chip_GPIO_SetPinState(LPC_GPIO_PORT, 0, Button3, false);

   Chip_GPIO_SetPinDIRInput(LPC_GPIO_PORT, 0, Button1);
   Chip_GPIO_SetPinDIRInput(LPC_GPIO_PORT, 0, Button2);
   Chip_GPIO_SetPinDIRInput(LPC_GPIO_PORT, 0, Button3);

   Chip_IOCON_PinSetMode(
   ((LPC_IOCON_T *) LPC_IOCON_BASE), BUTTON1_PIO, PIN_MODE_PULLUP);
   Chip_IOCON_PinSetMode(
   ((LPC_IOCON_T *) LPC_IOCON_BASE), BUTTON2_PIO, PIN_MODE_PULLUP);
   Chip_IOCON_PinSetMode(
   ((LPC_IOCON_T *) LPC_IOCON_BASE), BUTTON3_PIO, PIN_MODE_PULLUP);

#else

#endif
}

//---------------------------------------------------------------------
bool HW_Get_Button_State(Button button)
{
#ifdef Board_Demo_v1_4
   return !Chip_GPIO_GetPinState(LPC_GPIO_PORT, 0, button);
#else
   return false;
#endif

}


//---------------------------------------------------------------------
void HW_getTemp(uint8_t Buffer[])
{
#ifdef Board_Demo_v1_4
  uint8_t RX_Buffer[HAL_I2C_RX_RESERVED_BYTES + 2];
  uint8_t TX_Buffer[HAL_I2C_TX_RESERVED_BYTES + 1];

  /* get temperature value from sensor */
  TX_Buffer[HAL_I2C_TX_RESERVED_BYTES + 0] = 0;
  HAL_I2C_SendBytes(i2cHandleMaster, TEMP_I2C_ADDRESS >> 1, TX_Buffer, 1);
  HAL_I2C_RecvBytes(i2cHandleMaster, TEMP_I2C_ADDRESS >> 1, RX_Buffer, 2);

  Buffer[0] = RX_Buffer[HAL_I2C_RX_RESERVED_BYTES + 0];
  Buffer[1] = RX_Buffer[HAL_I2C_TX_RESERVED_BYTES + 1];
#else
  Buffer[0] = 0;
  Buffer[1] = 0;
   return;
#endif
}


