#include <string.h>
#include "board.h"
#include <stdlib.h>

#include "global_variables.h"
#include "HW_functions.h"
#include "crc32.h"

#include "ndef_message.h"
#define LED1 17
#define LED2 13
#define LED3 16
#define LED4 15
#define LED5 7
#define LED6 6
#define LED7 14
#define LED8 9
#define LED9 8

//---------------------------------------------------------------------
//               Helping functions declarations
//---------------------------------------------------------------------

//---------------------------------------------------------------------
///
/// Setup the Chip
void Setup();
//---------------------------------------------------------------------
///
/// Finds the start of the NDEF Message
/// @param buffer Buffer in which is searched
/// @param buffer_len length of the buffer
short int findNDEFStartIdx(uint8_t *buffer, uint32_t buffer_len);

//---------------------------------------------------------------------
///
/// Let a LED blink (blocking function ~ 700ms)
/// @param LED LED to blink
void LED_blink(LED led);

//---------------------------------------------------------------------
///
/// Checks Buttons and gives them back as a bitfield
/// @param Buttons Bitfield in which the button state is written(first Button = LSB, true = pressed, false = not pressed)
void check_Buttons(uint8_t *Buttons);

//---------------------------------------------------------------------
///
/// Writes the Timing in the User Memory formated as NDEF Message
/// @param time1 first time to write
/// @param time2 second time to write
short int write_back_Time_as_NDEF(int time1, int time2);

//---------------------------------------------------------------------
///
/// Writes the Android Application Record of the NTAG I2C Demo Android App
/// in the User Memory
bool reset_AAR();

//---------------------------------------------------------------------
//               Helping functions declarations end
//---------------------------------------------------------------------


//---------------------------------------------------------------------
///
/// Performs the Speedtest
/// @return returns an NTAG_ERROR_CODE
short int speedTest();


//---------------------------------------------------------------------
///

void led_blink(int led_pin) {

	for (int j = 0; j <3; j++) {

		Chip_GPIO_SetPinState(LPC_GPIO_PORT, 0, led_pin, true);

		for (int i = 0; i <1000000; i++) {
		}

		Chip_GPIO_SetPinState(LPC_GPIO_PORT, 0, led_pin, false);


		for (int i = 0; i <1000000; i++) {
		}
	}

}

void led_off() {

	Chip_GPIO_SetPinState(LPC_GPIO_PORT, 0, 7, true);
	Chip_GPIO_SetPinState(LPC_GPIO_PORT, 0, 16, true);
	Chip_GPIO_SetPinState(LPC_GPIO_PORT, 0, 17, true);

	Chip_GPIO_SetPinState(LPC_GPIO_PORT, 0, 14, false);
	Chip_GPIO_SetPinState(LPC_GPIO_PORT, 0, 13, false);
	Chip_GPIO_SetPinState(LPC_GPIO_PORT, 0, 0, false);



}


void pattern_1() {

	// horizontal wave
	for (int i =0; i < 3; i++) {


		Chip_GPIO_SetPinState(LPC_GPIO_PORT, 0, LED1, true);

		//wait
		for (int i = 0; i <100000; i++) {
		}

		Chip_GPIO_SetPinState(LPC_GPIO_PORT, 0, LED1, false);
		Chip_GPIO_SetPinState(LPC_GPIO_PORT, 0, LED4, true);

		for (int i = 0; i <100000; i++) {
		}

		Chip_GPIO_SetPinState(LPC_GPIO_PORT, 0, LED4, false);
		Chip_GPIO_SetPinState(LPC_GPIO_PORT, 0, LED7, true);

		for (int i = 0; i <100000; i++) {
		}

		Chip_GPIO_SetPinState(LPC_GPIO_PORT, 0, LED7, false);
		Chip_GPIO_SetPinState(LPC_GPIO_PORT, 0, LED2, true);

		for (int i = 0; i <100000; i++) {
		}

		Chip_GPIO_SetPinState(LPC_GPIO_PORT, 0, LED2, false);
		Chip_GPIO_SetPinState(LPC_GPIO_PORT, 0, LED5, true);

		for (int i = 0; i <100000; i++) {
		}

		Chip_GPIO_SetPinState(LPC_GPIO_PORT, 0, LED5, false);
		Chip_GPIO_SetPinState(LPC_GPIO_PORT, 0, LED8, true);

		for (int i = 0; i <100000; i++) {
		}

		Chip_GPIO_SetPinState(LPC_GPIO_PORT, 0, LED8, false);
		Chip_GPIO_SetPinState(LPC_GPIO_PORT, 0, LED3, true);

		for (int i = 0; i <100000; i++) {
		}

		Chip_GPIO_SetPinState(LPC_GPIO_PORT, 0, LED3, false);
		Chip_GPIO_SetPinState(LPC_GPIO_PORT, 0, LED6, true);

		for (int i = 0; i <100000; i++) {
		}

		Chip_GPIO_SetPinState(LPC_GPIO_PORT, 0, LED6, false);
		Chip_GPIO_SetPinState(LPC_GPIO_PORT, 0, LED9, true);

		for (int i = 0; i <100000; i++) {
		}

		Chip_GPIO_SetPinState(LPC_GPIO_PORT, 0, LED9, false);
	}


}


void pattern_2() {

	// vertical wave
	for (int i = 0; i < 3; i++) {

		Chip_GPIO_SetPinState(LPC_GPIO_PORT, 0, LED1, true);

		//wait
		for (int i = 0; i <100000; i++) {
		}

		Chip_GPIO_SetPinState(LPC_GPIO_PORT, 0, LED1, false);
		Chip_GPIO_SetPinState(LPC_GPIO_PORT, 0, LED2, true);

		for (int i = 0; i <100000; i++) {
		}

		Chip_GPIO_SetPinState(LPC_GPIO_PORT, 0, LED2, false);
		Chip_GPIO_SetPinState(LPC_GPIO_PORT, 0, LED3, true);

		for (int i = 0; i <100000; i++) {
		}

		Chip_GPIO_SetPinState(LPC_GPIO_PORT, 0, LED3, false);
		Chip_GPIO_SetPinState(LPC_GPIO_PORT, 0, LED4, true);

		for (int i = 0; i <100000; i++) {
		}

		Chip_GPIO_SetPinState(LPC_GPIO_PORT, 0, LED4, false);
		Chip_GPIO_SetPinState(LPC_GPIO_PORT, 0, LED5, true);

		for (int i = 0; i <100000; i++) {
		}

		Chip_GPIO_SetPinState(LPC_GPIO_PORT, 0, LED5, false);
		Chip_GPIO_SetPinState(LPC_GPIO_PORT, 0, LED6, true);

		for (int i = 0; i <100000; i++) {
		}

		Chip_GPIO_SetPinState(LPC_GPIO_PORT, 0, LED6, false);
		Chip_GPIO_SetPinState(LPC_GPIO_PORT, 0, LED7, true);

		for (int i = 0; i <100000; i++) {
		}

		Chip_GPIO_SetPinState(LPC_GPIO_PORT, 0, LED7, false);
		Chip_GPIO_SetPinState(LPC_GPIO_PORT, 0, LED8, true);

		for (int i = 0; i <100000; i++) {
		}

		Chip_GPIO_SetPinState(LPC_GPIO_PORT, 0, LED8, false);
		Chip_GPIO_SetPinState(LPC_GPIO_PORT, 0, LED9, true);

		for (int i = 0; i <100000; i++) {
		}

		Chip_GPIO_SetPinState(LPC_GPIO_PORT, 0, LED9, false);
	}
}


void pattern_3() {

	// 3 blinks

	for (int i = 0; i < 5; i++) {

		Chip_GPIO_SetPinState(LPC_GPIO_PORT, 0, LED1, true);
		Chip_GPIO_SetPinState(LPC_GPIO_PORT, 0, LED2, true);
		Chip_GPIO_SetPinState(LPC_GPIO_PORT, 0, LED3, true);

		//wait
		for (int i = 0; i <100000; i++) {
		}

		Chip_GPIO_SetPinState(LPC_GPIO_PORT, 0, LED1, false);
		Chip_GPIO_SetPinState(LPC_GPIO_PORT, 0, LED2, false);
		Chip_GPIO_SetPinState(LPC_GPIO_PORT, 0, LED3, false);

		Chip_GPIO_SetPinState(LPC_GPIO_PORT, 0, LED4, true);
		Chip_GPIO_SetPinState(LPC_GPIO_PORT, 0, LED5, true);
		Chip_GPIO_SetPinState(LPC_GPIO_PORT, 0, LED6, true);

		for (int i = 0; i <100000; i++) {
		}

		Chip_GPIO_SetPinState(LPC_GPIO_PORT, 0, LED4, false);
		Chip_GPIO_SetPinState(LPC_GPIO_PORT, 0, LED5, false);
		Chip_GPIO_SetPinState(LPC_GPIO_PORT, 0, LED6, false);

		Chip_GPIO_SetPinState(LPC_GPIO_PORT, 0, LED7, true);
		Chip_GPIO_SetPinState(LPC_GPIO_PORT, 0, LED8, true);
		Chip_GPIO_SetPinState(LPC_GPIO_PORT, 0, LED9, true);

		for (int i = 0; i <100000; i++) {
		}

		Chip_GPIO_SetPinState(LPC_GPIO_PORT, 0, LED7, false);
		Chip_GPIO_SetPinState(LPC_GPIO_PORT, 0, LED8, false);
		Chip_GPIO_SetPinState(LPC_GPIO_PORT, 0, LED9, false);
	}

}

/// main Program
/// @return should never return
int main(void) {
	Setup();

	// Initialize main buffer used to read and write user memory
	uint8_t rxbuffer[4 * NFC_BLOCK_SIZE];
	uint8_t txbuffer[2 * NFC_BLOCK_SIZE];
	memset(rxbuffer, 0, 4 * NFC_BLOCK_SIZE);
	memset(txbuffer, 0, 2 * NFC_BLOCK_SIZE);

	//Chip_GPIO_SetPinDIROutput(LPC_GPIO_PORT, 0, 0);
    //Chip_GPIO_SetPinState(LPC_GPIO_PORT, 0, 0, false);


	// prepare defined state for SRAM
	NFC_DisableSRAM(ntag_handle);

	// If Button1 is pressed, read NDEF message from EEPROM and light up according LED color
	uint8_t Buttons = 0;


#ifdef INTERRUPT
	// If Interrupted Mode is enabled set the FD Pin to react on the SRAM
	NFC_WriteRegister(ntag_handle, NTAG_MEM_OFFSET_NC_REG,
	NTAG_NC_REG_MASK_FD_OFF | NTAG_NC_REG_MASK_FD_ON,
	NTAG_NC_REG_MASK_FD_OFF | NTAG_NC_REG_MASK_FD_ON);
#endif

	Chip_Clock_EnablePeriphClock(SYSCTL_CLOCK_GPIO);
	/**
	Chip_GPIO_SetPinDIROutput(LPC_GPIO_PORT, 0, 0);
	Chip_GPIO_SetPinDIROutput(LPC_GPIO_PORT, 0, 14);
	Chip_GPIO_SetPinDIROutput(LPC_GPIO_PORT, 0, 13);
	Chip_GPIO_SetPinDIROutput(LPC_GPIO_PORT, 0, 8);




	Chip_GPIO_SetPinState(LPC_GPIO_PORT, 0, 14, true);
	Chip_GPIO_SetPinState(LPC_GPIO_PORT, 0, 13, true);
	Chip_GPIO_SetPinState(LPC_GPIO_PORT, 0, 0, true);
	Chip_GPIO_SetPinState(LPC_GPIO_PORT, 0, 8, true);




	Chip_GPIO_SetPinState(LPC_GPIO_PORT, 0, 14, false);
	Chip_GPIO_SetPinState(LPC_GPIO_PORT, 0, 13, false);
	Chip_GPIO_SetPinState(LPC_GPIO_PORT, 0, 0, false);
	Chip_GPIO_SetPinState(LPC_GPIO_PORT, 0, 8, false);
	**/

	// set all pins to output
	Chip_GPIO_SetPinDIROutput(LPC_GPIO_PORT, 0, LED1);
	Chip_GPIO_SetPinDIROutput(LPC_GPIO_PORT, 0, LED2);
	Chip_GPIO_SetPinDIROutput(LPC_GPIO_PORT, 0, LED3);
	Chip_GPIO_SetPinDIROutput(LPC_GPIO_PORT, 0, LED4);
	Chip_GPIO_SetPinDIROutput(LPC_GPIO_PORT, 0, LED5);
	Chip_GPIO_SetPinDIROutput(LPC_GPIO_PORT, 0, LED6);
	Chip_GPIO_SetPinDIROutput(LPC_GPIO_PORT, 0, LED7);
	Chip_GPIO_SetPinDIROutput(LPC_GPIO_PORT, 0, LED8);
	Chip_GPIO_SetPinDIROutput(LPC_GPIO_PORT, 0, LED9);

	// set all pins to high
	Chip_GPIO_SetPinState(LPC_GPIO_PORT, 0, LED1, true);
	Chip_GPIO_SetPinState(LPC_GPIO_PORT, 0, LED2, true);
	Chip_GPIO_SetPinState(LPC_GPIO_PORT, 0, LED3, true);
	Chip_GPIO_SetPinState(LPC_GPIO_PORT, 0, LED4, true);
	Chip_GPIO_SetPinState(LPC_GPIO_PORT, 0, LED5, true);
	Chip_GPIO_SetPinState(LPC_GPIO_PORT, 0, LED6, true);
	Chip_GPIO_SetPinState(LPC_GPIO_PORT, 0, LED7, true);
	Chip_GPIO_SetPinState(LPC_GPIO_PORT, 0, LED8, true);
	Chip_GPIO_SetPinState(LPC_GPIO_PORT, 0, LED9, true);

	// set all pins to low
	Chip_GPIO_SetPinState(LPC_GPIO_PORT, 0, LED1, false);
	Chip_GPIO_SetPinState(LPC_GPIO_PORT, 0, LED2, false);
	Chip_GPIO_SetPinState(LPC_GPIO_PORT, 0, LED3, false);
	Chip_GPIO_SetPinState(LPC_GPIO_PORT, 0, LED4, false);
	Chip_GPIO_SetPinState(LPC_GPIO_PORT, 0, LED5, false);
	Chip_GPIO_SetPinState(LPC_GPIO_PORT, 0, LED6, false);
	Chip_GPIO_SetPinState(LPC_GPIO_PORT, 0, LED7, false);
	Chip_GPIO_SetPinState(LPC_GPIO_PORT, 0, LED8, false);
	Chip_GPIO_SetPinState(LPC_GPIO_PORT, 0, LED9, false);




	//HW_switchLEDs(GREENLED);





	//led_off();


    //Chip_GPIO_SetPinState(LPC_GPIO_PORT, 0, 14, true);
	//led_blink(17);




	// Main Loop
	while (1) {
		// Enable Passthrough Mode RF->I2C


		// wait for RF Write in the SRAM terminator page
		while (NFC_WaitForEvent(ntag_handle, NTAG_EVENT_RF_WROTE_SRAM,
		SRAM_TIMEOUT)) {

			// check if PT is off(happens when NTAG is not in the field anymore)
			// and switch it back on
			uint8_t reg = 0;
			NFC_ReadRegister(ntag_handle, NFC_MEM_OFFSET_NC_REG, &reg);
			if (!(reg & NFC_NC_REG_MASK_TR_SRAM_ON_OFF)) {
				NFC_SetPassThroughRFtoI2C_withEn(ntag_handle);
			}
		}

		// get the SRAM Data
		memset(sram_buf, 0, NFC_MEM_SIZE_SRAM);
		NFC_ReadBytes(ntag_handle, NFC_MEM_START_ADDR_SRAM, sram_buf,
		NFC_MEM_SIZE_SRAM);

		// Check for requested action Speedtest or LEDDemo
		// ---- Speedtest ----
		if (sram_buf[NFC_MEM_SIZE_SRAM - 4] == 'S') {
			// run speed test
			error_code = speedTest();

			// let the Green LED blink if Speedtest was successful
			if (error_code) {
				HW_switchLEDs(LEDOFF);
			} else {
				LED_blink(GREENLED);
			}
		}
		// ---- LedDemo ----
		else if (sram_buf[NFC_MEM_SIZE_SRAM - 4] == 'L') {
			// turn on the according LED
			if (sram_buf[NFC_MEM_SIZE_SRAM - 3] == '1') {
				//HW_switchLEDs(REDLED);

				//led_blink(0);
				//test for new board
				//led_blink(LED7);
				pattern_1();
			}

			if (sram_buf[NFC_MEM_SIZE_SRAM - 3] == '2') {
				//HW_switchLEDs(BLUELED);
				//led_blink(16);

				//led_blink(14);
				//led_blink(LED7);
				pattern_2();
			}
			if (sram_buf[NFC_MEM_SIZE_SRAM - 3] == '3') {
				//HW_switchLEDs(REDLED);

				//led_blink(13);
				//led_blink(LED7);
				pattern_3();


			}
			if (sram_buf[NFC_MEM_SIZE_SRAM - 3] == '0') {
				//HW_switchLEDs(LEDOFF);
				led_off();
			}

			// Check Buttons and Temperature, put data in SRAM buffer
			uint8_t Temperature[2];
			//check_Buttons(&Buttons);
			//HW_getTemp(Temperature);

			//sram_buf[NFC_MEM_SIZE_SRAM - 6] = Temperature[0];
			//sram_buf[NFC_MEM_SIZE_SRAM - 5] = Temperature[1];

			//sram_buf[NFC_MEM_SIZE_SRAM - 2] = Buttons;

			// write back Data
			//NFC_SetPassThroughI2CtoRF(ntag_handle);
			//NFC_WriteBytes(ntag_handle, NFC_MEM_START_ADDR_SRAM, sram_buf,
			//NFC_MEM_SIZE_SRAM);

			// waiting till RF has read
			//while (NFC_WaitForEvent(ntag_handle, NTAG_EVENT_RF_READ_SRAM, 10))
				;
		} else if (sram_buf[NFC_MEM_SIZE_SRAM - 4] == 'V')
		// ---- Version information ----
				{

			memset(sram_buf, 0, NFC_MEM_SIZE_SRAM);
			int index = 0;
			memcpy(&sram_buf[index], "Board Ver.: ", 12);
			index = 12;
			sram_buf[index++] = CHAR_BOARD_MAJ;
			sram_buf[index++] = '.';
			sram_buf[index++] = CHAR_BOARD_MIN;
			sram_buf[index++] = '\n';

			memcpy(&sram_buf[index], "FW    Ver.: ", 12);
			index += 12;
			sram_buf[index++] = CHAR_FW_MAJ;
			sram_buf[index++] = '.';
			sram_buf[index++] = CHAR_FW_MIN;
			sram_buf[index++] = '\n';

			// write back Version information
			NFC_SetPassThroughI2CtoRF(ntag_handle);
			NFC_WriteBytes(ntag_handle, NFC_MEM_START_ADDR_SRAM, sram_buf,
			NFC_MEM_SIZE_SRAM);

			// waiting till RF has read
			while (NFC_WaitForEvent(ntag_handle, NTAG_EVENT_RF_READ_SRAM, 10))
				;
		}
	}

// never leave this function
	while (1) {
		__WFE();
	}
	return 0;
}

//---------------------------------------------------------------------
///
/// performs the Speedtest
short int speedTest() {
	uint32_t start_reader;
	uint32_t stop_reader;

	uint32_t start_tag;
	uint32_t stop_tag;

	int counter = 0;
	memset(sram_buf, 0, NFC_MEM_SIZE_SRAM);
	error_code = NFC_EnableSRAM(ntag_handle);

	if (error_code != NTAG_ERR_OK)
		return error_code;

	// save start timer
	start_reader = HAL_Timer_getTime_ms();

	// Reset the seed to its initial value for the CRC32 calculation
	uint32_t seed = 0xFFFFFFFF;
	uint32_t crcRx = 0;

	int lastBlock = true;
	// Begin to Read Data
	do {

		//----------------------------------------------
		// wait for data write by RF
		if (NFC_WaitForEvent(ntag_handle, NTAG_EVENT_RF_WROTE_SRAM,
		SRAM_TIMEOUT))
			return NTAG_ERR_COMMUNICATION;

		//----------------------------------------------
		// data is ready, read whole SRAM
		error_code = NFC_ReadBytes(ntag_handle, NFC_MEM_START_ADDR_SRAM,
				sram_buf,
				NFC_MEM_SIZE_SRAM);
		counter++;

		// Set LED
		HW_switchLEDs((counter % 3) + 1);

		// The last block contains the "finish_S" string to notify the microcontroller about the message transmission completion
		lastBlock = strncmp((char*) &sram_buf[0], "finish_S", 8);

		// Update content for the CRC32 calculation
		if (lastBlock != 0) {
			crcRx = crc32(seed, sram_buf, NFC_MEM_SIZE_SRAM);
			seed = crcRx;
		} else {
			crcRx = crc32(seed, sram_buf, NFC_MEM_SIZE_SRAM - 4);
		}
	} while (lastBlock != 0);

	// Get the CRC values in reception
	uint32_t crcReceived = (uint8_t) sram_buf[NFC_MEM_SIZE_SRAM - 1] << 24
			| (uint8_t) sram_buf[NFC_MEM_SIZE_SRAM - 2] << 16
			| (uint8_t) sram_buf[NFC_MEM_SIZE_SRAM - 3] << 8
			| (uint8_t) sram_buf[NFC_MEM_SIZE_SRAM - 4];

	// save stop timer
	stop_reader = HAL_Timer_getTime_ms();

	// switch PT direction I2C -> RF
	NFC_SetPassThroughI2CtoRF(ntag_handle);

	//clear sram Buffer
	memset(sram_buf, 0, NFC_MEM_SIZE_SRAM);

	int pack = counter;

	// save start timer
	start_tag = HAL_Timer_getTime_ms();

	// Reset the seed to its initial value for the CRC32 calculation
	seed = 0xFFFFFFFF;
	uint32_t crcTx = 0;

	// Begin to Write Data
	for (; counter != 0; counter--) {
		// write number do identify blocks
		sram_buf[0] = pack - counter;

		// for last block write, write finish identifier
		// The last block contains the "finish_S" string to notify the LPC board about the message transmission completion
		if (counter == 1) {
			memcpy(sram_buf, "finish_S", 8);

			// Update content for the CRC32 calculation (last 4 bytes are the CRC32)
			crcTx = crc32(seed, sram_buf, NFC_MEM_SIZE_SRAM - 4);

			// Write the result of the prev CRC check
			if (crcRx != crcReceived)
				sram_buf[NFC_MEM_SIZE_SRAM - 5] = 0x01;
			else
				sram_buf[NFC_MEM_SIZE_SRAM - 5] = 0x00;

			// Append the CRC32 to the block as this is the last block to be transmitted
			sram_buf[NFC_MEM_SIZE_SRAM - 4] = (crcTx >> 0) & 0xFF;
			sram_buf[NFC_MEM_SIZE_SRAM - 3] = (crcTx >> 8) & 0xFF;
			sram_buf[NFC_MEM_SIZE_SRAM - 2] = (crcTx >> 16) & 0xFF;
			sram_buf[NFC_MEM_SIZE_SRAM - 1] = (crcTx >> 24) & 0xFF;
		} else {
			// Update content for the CRC32 calculation
			crcTx = crc32(seed, sram_buf, NFC_MEM_SIZE_SRAM);
			seed = crcTx;
		}

		//----------------------------------------------
		// Write Data to SRAM
		error_code = NFC_WriteBytes(ntag_handle, NFC_MEM_START_ADDR_SRAM,
				sram_buf,
				NFC_MEM_SIZE_SRAM);

		//----------------------------------------------
		// wait for data read
		if (NFC_WaitForEvent(ntag_handle, NTAG_EVENT_RF_READ_SRAM,
		SRAM_TIMEOUT))
			return NTAG_ERR_COMMUNICATION;

		//SetLED
		HW_switchLEDs((counter % 3) + 1);

	}

	// stop time
	stop_tag = HAL_Timer_getTime_ms();

	// calculate Time
	uint32_t time1 = stop_reader - start_reader;
	uint32_t time2 = stop_tag - start_tag;


#ifdef WRITE_TIME_VIA_NDEF
	// Writes the Time in the UserMemory as a NDEF Message
	write_back_Time_as_NDEF(time1, time2);
#endif

	// Check the integrity of the data received (not done before to avoid stopping the execution of the rest of the SpeedTest demo)
	if (crcRx != crcReceived)
		return NTAG_ERR_COMMUNICATION;

	return NTAG_ERR_OK;
}

//---------------------------------------------------------------------
//               Helping functions begin
//---------------------------------------------------------------------

//---------------------------------------------------------------------
void Setup() {
	HW_switchLEDs(LEDOFF);

	// Initialize peripherals
	HAL_BSP_BoardInit();

	// Setup Pins on the microcontroller
	HW_setup_Board_for_use_with_NTAG();

	// Setup the Buttons
	HW_init_Buttons();

	// enable delay timer
	HAL_Timer_Init();

	// Initialize the Interrupt Service Routine
	HAL_ISR_Init();

	// Initialize I2C
	i2cHandleMaster = HAL_I2C_InitDevice(HAL_I2C_INIT_DEFAULT);
	SystemCoreClockUpdate();

	// Set interrupt for time measurement
	SysTick_Config(SystemCoreClock / 1000); // produce a timer interrupt every 1ms

	// Initialize the NTAG I2C components
	ntag_handle = NFC_InitDevice(NFC_TEST_DEVICE, i2cHandleMaster);
	HAL_ISR_RegisterCallback(0, ISR_LEVEL_LO, NULL, NULL);

	// Enable IRQ for BOD
	Chip_SYSCTL_EnableBODReset();
}

//---------------------------------------------------------------------
short int findNDEFStartIdx(uint8_t *buffer, uint32_t buffer_len) {
	short int idx = 0;
	if (buffer_len < 11)
		return -1;

	if (buffer == NULL)
		return -1;

	// find first valid NDEF message in buffer
	// header is 10 bytes long, so an empty message will have at
	// least 11 bytes
	for (idx = 0; idx < (buffer_len - 10); idx++) {
		if (buffer[idx] == 0x03 && (buffer[idx + 1] + idx + 2) < buffer_len
				&& buffer[buffer[idx + 1] + idx + 2] == 0xFE) {
			break;
		}
	}
	if (idx > (buffer_len - 11))
		return -1;

	idx += 9; // if length < 255

	return idx;
}

//---------------------------------------------------------------------
void LED_blink(LED led) {
	HW_switchLEDs(LEDOFF);
	HAL_Timer_delay_ms(100);
	HW_switchLEDs(led);
	HAL_Timer_delay_ms(100);
	HW_switchLEDs(LEDOFF);
	HAL_Timer_delay_ms(100);
	HW_switchLEDs(led);
	HAL_Timer_delay_ms(100);
	HW_switchLEDs(LEDOFF);
	HAL_Timer_delay_ms(100);
	HW_switchLEDs(led);
	HAL_Timer_delay_ms(100);
	HW_switchLEDs(LEDOFF);
	HAL_Timer_delay_ms(100);

}

//---------------------------------------------------------------------
void check_Buttons(uint8_t *Buttons) {
	if (HW_Get_Button_State(Button1)) {
		*Buttons |= 0x01;
	} else {
		*Buttons &= ~0x01;
	}

	if (HW_Get_Button_State(Button2)) {
		*Buttons |= 0x02;
	} else {
		*Buttons &= ~0x02;
	}

	if (HW_Get_Button_State(Button3)) {
		*Buttons |= 0x04;
	} else {
		*Buttons &= ~0x04;
	}
}

//---------------------------------------------------------------------
short int write_back_Time_as_NDEF(int time1, int time2) {
//check for overflow
	if (time1 >= 99999)
		time1 = 99999;
	if (time2 >= 99999)
		time2 = 99999;

//----------------------------------------------
// write back times as ndef
	int index = 0;

// Header of the NDEF Message
	sram_buf[index++] = 0x03;
	sram_buf[index++] = 0x1C; // Message length
	sram_buf[index++] = 0xd1;

	sram_buf[index++] = 0x01; // Text Record
	sram_buf[index++] = 0x18; // Record length
	sram_buf[index++] = 0x54;
	sram_buf[index++] = 0x02;

	sram_buf[index++] = 0x65;
	sram_buf[index++] = 0x6e;

	char temp[5];
	uint8_t k;

	memset(temp, 0, 5);
	itoa(time1, temp, 10);

	sram_buf[index++] = '(';

	for (k = 0; k < 5 - strlen(temp); k++)
		sram_buf[index++] = '0';

	for (k = 0; k < strlen(temp); k++)
		sram_buf[index++] = temp[k];

	sram_buf[index++] = ')';

	memset(temp, 0, 5);
	itoa(time2, temp, 10);

	sram_buf[index++] = '(';

	for (k = 0; k < 5 - strlen(temp); k++)
		sram_buf[index++] = '0';

	for (k = 0; k < strlen(temp); k++)
		sram_buf[index++] = temp[k];

	sram_buf[index++] = ')';

	memset(temp, 0, 5);

	sram_buf[index++] = '(';

	for (k = 0; k < 5 - strlen(temp); k++)
		sram_buf[index++] = '0';

	for (k = 0; k < strlen(temp); k++)
		sram_buf[index++] = temp[k];

	sram_buf[index++] = ')';

// end of NDEF Message
	sram_buf[index++] = 0xFE;
	sram_buf[index++] = 0x00;

// Disable the PT
	error_code = NFC_DisableSRAM(ntag_handle);

// get the timeout time
	int current_loop = HAL_Timer_getTime_ms() + SRAM_TIMEOUT;

	uint8_t reg = 0;
// write back the Result in EEPROM as NDEF
	do {
		error_code = NFC_WriteBytes(ntag_handle, NFC_MEM_START_ADDR_USER_MEMORY,
				sram_buf, index);

		NFC_ReadRegister(ntag_handle, NFC_MEM_OFFSET_NS_REG, &reg);
		while (reg & NFC_NS_REG_MASK_I2C_IF_ON_OFF) {
			NFC_ReadRegister(ntag_handle, NFC_MEM_OFFSET_NS_REG, &reg);
		}

		if (current_loop < HAL_Timer_getTime_ms())
			return NTAG_ERR_COMMUNICATION;

	} while (error_code != NTAG_ERR_OK);

	return NTAG_ERR_OK;
}

//---------------------------------------------------------------------
bool reset_AAR() {

	error_code = NFC_WriteBytes(ntag_handle, NFC_MEM_START_ADDR_USER_MEMORY,
			Default_NDEF_Message, Default_NDEF_Message_length);

	return error_code;
}

//---------------------------------------------------------------------
//               Helping functions end
//---------------------------------------------------------------------

